@@ -92,7 +92,7 @@ is_supported_extension (GtkWidget *file_sel,
 {
 	int i;
 	for (i = 0; file_type[i] != -1; i++)
-		if (file_extension_is (filename, mime_type_desc[file_type[i]].default_ext))
+		if (_g_filename_has_extension (filename, mime_type_desc[file_type[i]].default_ext))
 			return TRUE;
 	return FALSE;
 }
@@ -111,7 +111,7 @@ get_full_uri (DlgNewData *data)
 	if ((uri == NULL) || (*uri == 0))
 		return NULL;
 
-	filename = file_name_from_path (uri);
+	filename = _g_path_get_file_name (uri);
 	if ((filename == NULL) || (*filename == 0)) {
 		g_free (uri);
 		return NULL;
@@ -124,7 +124,7 @@ get_full_uri (DlgNewData *data)
 
 		uri_ext = get_archive_filename_extension (uri);
 		default_ext = mime_type_desc[data->supported_types[idx-1]].default_ext;
-		if (strcmp_null_tolerant (uri_ext, default_ext) != 0) {
+		if (_g_strcmp_null_tolerant (uri_ext, default_ext) != 0) {
 			full_uri = g_strconcat (uri, default_ext, NULL);
 			g_free (uri);
 		}
@@ -221,7 +221,7 @@ get_archive_filename_from_selector (DlgNewData *data)
 
 	debug (DEBUG_INFO, "create/save %s\n", uri);
 
-	if (uri_exists (uri)) {
+	if (_g_uri_query_exists (uri)) {
 		GtkWidget *dialog;
 
 		if (! is_supported_extension (data->dialog, uri, data->supported_types)) {
@@ -279,7 +279,7 @@ new_file_response_cb (GtkWidget  *w,
 	}
 
 	if (response == GTK_RESPONSE_HELP) {
-		show_help_dialog (GTK_WINDOW (data->dialog), "archive-create");
+		_gtk_show_help_dialog (GTK_WINDOW (data->dialog), "archive-create");
 		return;
 	}
 
@@ -434,7 +434,7 @@ save_file_response_cb (GtkWidget  *w,
 	}
 
 	if (response == GTK_RESPONSE_HELP) {
-		show_help_dialog (GTK_WINDOW (data->dialog), "archive-convert");
+		_gtk_show_help_dialog (GTK_WINDOW (data->dialog), "archive-convert");
 		return;
 	}
 
@@ -705,7 +705,7 @@ activate_action_view_or_open (GtkAction *action,
 	if (file_list == NULL)
 		return;
 	fr_window_open_files (window, file_list, FALSE);
-	path_list_free (file_list);
+	_g_string_list_free (file_list);
 }
 
 
@@ -842,7 +842,7 @@ activate_action_manual (GtkAction *action,
 {
 	FrWindow *window = data;
 
-	show_help_dialog (GTK_WINDOW (window) , NULL);
+	_gtk_show_help_dialog (GTK_WINDOW (window) , NULL);
 }
 
 
