@@ -27,6 +27,7 @@
 #include "file-utils.h"
 #include "fr-stock.h"
 #include "fr-window.h"
+#include "glib-utils.h"
 #include "gtk-utils.h"
 #include "preferences.h"
 
@@ -80,14 +81,14 @@ file_sel_response_cb (GtkWidget      *widget,
 	}
 
 	if (response == GTK_RESPONSE_HELP) {
-		show_help_dialog (GTK_WINDOW (data->dialog), "archive-edit");
+		_gtk_show_help_dialog (GTK_WINDOW (data->dialog), "archive-edit");
 		g_free (current_folder);
 		return TRUE;
 	}
 
 	/* check folder permissions. */
 
-	if (uri_is_dir (current_folder) && ! check_permissions (current_folder, R_OK)) {
+	if (_g_uri_query_is_dir (current_folder) && ! _g_uri_check_permissions (current_folder, R_OK)) {
 		GtkWidget *d;
 		char      *utf8_path;
 
@@ -120,7 +121,7 @@ file_sel_response_cb (GtkWidget      *widget,
 	if (item_list != NULL)
 		fr_window_archive_add_files (window, item_list, update);
 
-	gio_file_list_free (item_list);
+	_g_file_list_free (item_list);
 	g_slist_foreach (selections, (GFunc) g_free, NULL);
 	g_slist_free (selections);
 	g_free (current_folder);
