@@ -119,14 +119,6 @@ set_archive_options (DialogData *data)
 }
 
 
-static void
-help_clicked_cb (GtkWidget  *widget,
-		 DialogData *data)
-{
-	show_help_dialog (GTK_WINDOW (GET_WIDGET ("dialog")), "archive-create");
-}
-
-
 static void
 add_clicked_cb (GtkWidget  *widget,
 		DialogData *data)
@@ -163,7 +155,7 @@ add_clicked_cb (GtkWidget  *widget,
 
 		return;
 	}
-	else if (strchrs (archive_name, BAD_CHARS)) {
+	else if (_g_strchrs (archive_name, BAD_CHARS)) {
 		GtkWidget *d;
 		char      *utf8_name = g_filename_display_name (archive_name);
 
@@ -193,7 +185,7 @@ add_clicked_cb (GtkWidget  *widget,
 		return;
 	}
 
-	if (! check_permissions (archive_dir, R_OK|W_OK|X_OK)) {
+	if (! _g_uri_check_permissions (archive_dir, R_OK|W_OK|X_OK)) {
 		GtkWidget  *d;
 
 		d = _gtk_error_dialog_new (GTK_WINDOW (window),
@@ -210,7 +202,7 @@ add_clicked_cb (GtkWidget  *widget,
 		return;
 	}
 
-	if (! uri_is_dir (archive_dir)) {
+	if (! _g_uri_query_is_dir (archive_dir)) {
 		GtkWidget *d;
 		int        r;
 		char      *folder_name;
@@ -238,7 +230,7 @@ add_clicked_cb (GtkWidget  *widget,
 		do_not_add = (r != GTK_RESPONSE_YES);
 	}
 
-	if (! do_not_add && ! ensure_dir_exists (archive_dir, 0755, &error)) {
+	if (! do_not_add && ! _g_uri_ensure_dir_exists (archive_dir, 0755, &error)) {
 		GtkWidget  *d;
 
 		d = _gtk_error_dialog_new (GTK_WINDOW (window),
@@ -284,7 +276,7 @@ add_clicked_cb (GtkWidget  *widget,
 	g_free (tmp);
 	archive_file = g_strconcat (archive_dir, "/", archive_name, NULL);
 
-	if (uri_is_dir (archive_file)) {
+	if (_g_uri_query_is_dir (archive_file)) {
 		GtkWidget  *d;
 
 		d = _gtk_error_dialog_new (GTK_WINDOW (window),
@@ -303,7 +295,7 @@ add_clicked_cb (GtkWidget  *widget,
 		return;
 	}
 
-	if (uri_exists (archive_file)) {
+	if (_g_uri_query_exists (archive_file)) {
 		GtkWidget *d;
 		int        r;
 
@@ -398,7 +390,7 @@ archive_type_combo_box_changed_cb (GtkComboBox *combo_box,
 
 	mime_type = mime_type_desc[data->supported_types[idx]].mime_type;
 
-	icon = get_mime_type_pixbuf (mime_type, ARCHIVE_ICON_SIZE, NULL);
+	icon = _g_mime_type_get_icon (mime_type, ARCHIVE_ICON_SIZE, NULL);
 	if (icon != NULL) {
 		gtk_image_set_from_pixbuf (GTK_IMAGE (GET_WIDGET ("archive_icon_image")), icon);
 		g_object_unref (icon);
@@ -433,13 +425,23 @@ update_archive_type_combo_box_from_ext (DialogData  *data,
 static void
 update_sensitivity (DialogData *data)
 {
-	gtk_widget_set_sensitive (GET_WIDGET ("a_volume_spinbutton"), gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("a_volume_checkbutton"))));
+	const char *password;
+	gboolean    has_password;
+
+	gtk_widget_set_sensitive (GET_WIDGET ("a_volume_spinbutton"),
+				  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("a_volume_checkbutton"))));
+
+	password = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("a_password_entry")));
+	has_password = (password != NULL) && (*password != '\0');
+	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (GET_WIDGET ("a_encrypt_header_checkbutton")), ! has_password);
+	gtk_widget_set_sensitive (GET_WIDGET ("a_encrypt_header_checkbutton"), has_password);
 }
 
 
 static void
-password_entry_changed_cb (GtkEditable *editable,
-			   gpointer     user_data)
+password_entry_notify_text_cb (GObject    *object,
+			       GParamSpec *spec,
+			       gpointer    user_data)
 {
 	update_sensitivity ((DialogData *) user_data);
 }
@@ -479,7 +481,7 @@ dlg_batch_add_files (FrWindow *window,
 
 	data->window = window;
 	data->file_list = file_list;
-	data->single_file = ((file_list->next == NULL) && uri_is_file ((char*) file_list->data));
+	data->single_file = ((file_list->next == NULL) && _g_uri_query_is_file ((char*) file_list->data));
 	data->add_clicked = FALSE;
 
 	/* Set widgets data. */
@@ -496,25 +498,25 @@ dlg_batch_add_files (FrWindow *window,
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("a_volume_spinbutton")), (double) g_settings_get_int (data->settings, PREF_BATCH_ADD_VOLUME_SIZE) / MEGABYTE);
 
 	first_filename = (char*) file_list->data;
-	parent = remove_level_from_path (first_filename);
+	parent = _g_path_remove_level (first_filename);
 
 	if (file_list->next == NULL)
-		automatic_name = g_uri_unescape_string (file_name_from_path ((char*) file_list->data), NULL);
+		automatic_name = g_uri_unescape_string (_g_path_get_file_name ((char*) file_list->data), NULL);
 	else {
-		automatic_name = g_uri_unescape_string (file_name_from_path (parent), NULL);
+		automatic_name = g_uri_unescape_string (_g_path_get_file_name (parent), NULL);
 		if ((automatic_name == NULL) || (automatic_name[0] == '\0')) {
 			g_free (automatic_name);
-			automatic_name = g_uri_unescape_string (file_name_from_path (first_filename), NULL);
+			automatic_name = g_uri_unescape_string (_g_path_get_file_name (first_filename), NULL);
 		}
 	}
 
 	_gtk_entry_set_filename_text (GTK_ENTRY (GET_WIDGET ("a_add_to_entry")), automatic_name);
 	g_free (automatic_name);
 
-	if (check_permissions (parent, R_OK | W_OK))
+	if (_g_uri_check_permissions (parent, R_OK | W_OK))
 		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (GET_WIDGET ("a_location_filechooserbutton")), parent);
 	else
-		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (GET_WIDGET ("a_location_filechooserbutton")), get_home_uri ());
+		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (GET_WIDGET ("a_location_filechooserbutton")), _g_uri_get_home ());
 	g_free (parent);
 
 	/* archive type combobox */
@@ -547,17 +549,13 @@ dlg_batch_add_files (FrWindow *window,
 			  "clicked",
 			  G_CALLBACK (add_clicked_cb),
 			  data);
-	g_signal_connect (GET_WIDGET ("a_help_button"),
-			  "clicked",
-			  G_CALLBACK (help_clicked_cb),
-			  data);
 	g_signal_connect (G_OBJECT (data->archive_type_combo_box),
 			  "changed",
 			  G_CALLBACK (archive_type_combo_box_changed_cb),
 			  data);
 	g_signal_connect (GET_WIDGET ("a_password_entry"),
-			  "changed",
-			  G_CALLBACK (password_entry_changed_cb),
+			  "notify::text",
+			  G_CALLBACK (password_entry_notify_text_cb),
 			  data);
 	g_signal_connect (GET_WIDGET ("a_volume_checkbutton"),
 			  "toggled",
