@@ -37,6 +37,7 @@
 #include "fr-command.h"
 #include "fr-error.h"
 #include "fr-marshal.h"
+#include "fr-proc-error.h"
 #include "fr-process.h"
 #include "fr-init.h"
 
@@ -80,7 +81,7 @@ dropped_items_data_new (FrArchive     *archive,
 
 	data = g_new0 (DroppedItemsData, 1);
 	data->archive = archive;
-	data->item_list = path_list_dup (item_list);
+	data->item_list = _g_string_list_dup (item_list);
 	if (base_dir != NULL)
 		data->base_dir = g_strdup (base_dir);
 	if (dest_dir != NULL)
@@ -101,7 +102,7 @@ dropped_items_data_free (DroppedItemsData *data)
 {
 	if (data == NULL)
 		return;
-	path_list_free (data->item_list);
+	_g_string_list_free (data->item_list);
 	g_free (data->base_dir);
 	g_free (data->dest_dir);
 	g_free (data->password);
@@ -114,9 +115,6 @@ struct _FrArchivePrivData {
 							     * fr_archive_load is invoked, used
 							     * in batch mode. */
 	gpointer             fake_load_data;
-	FakeLoadFunc         add_is_stoppable_func;         /* Returns whether the add operation is
-							     * stoppable. */
-	gpointer             add_is_stoppable_data;
 	GCancellable        *cancellable;
 	char                *temp_dir;
 	gboolean             continue_adding_dropped_items;
@@ -154,7 +152,7 @@ xfer_data_free (XferData *data)
 
 	g_free (data->uri);
 	g_free (data->password);
-	path_list_free (data->file_list);
+	_g_string_list_free (data->file_list);
 	g_free (data->base_uri);
 	g_free (data->dest_dir);
 	g_free (data->tmp_dir);
@@ -249,10 +247,10 @@ fr_archive_class_init (FrArchiveClass *class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (FrArchiveClass, done),
 			      NULL, NULL,
-			      fr_marshal_VOID__INT_POINTER,
+			      fr_marshal_VOID__INT_BOXED,
 			      G_TYPE_NONE, 2,
 			      G_TYPE_INT,
-			      G_TYPE_POINTER);
+			      FR_TYPE_PROC_ERROR);
 	fr_archive_signals[PROGRESS] =
 		g_signal_new ("progress",
 			      G_TYPE_FROM_CLASS (class),
@@ -337,21 +335,11 @@ fr_archive_action_completed (FrArchive       *archive,
 }
 
 
-static gboolean
-fr_archive_add_is_stoppable (FrArchive *archive)
-{
-	if (archive->priv->add_is_stoppable_func != NULL)
-		return (*archive->priv->add_is_stoppable_func) (archive, archive->priv->add_is_stoppable_data);
-	else
-		return FALSE;
-}
-
-
 static gboolean
 archive_sticky_only_cb (FrProcess *process,
 			FrArchive *archive)
 {
-	fr_archive_stoppable (archive, fr_archive_add_is_stoppable (archive));
+	fr_archive_stoppable (archive, FALSE);
 	return TRUE;
 }
 
@@ -369,8 +357,6 @@ fr_archive_init (FrArchive *archive)
 	archive->priv = g_new0 (FrArchivePrivData, 1);
 	archive->priv->fake_load_func = NULL;
 	archive->priv->fake_load_data = NULL;
-	archive->priv->add_is_stoppable_func = NULL;
-	archive->priv->add_is_stoppable_data = NULL;
 
 	archive->priv->extraction_destination = NULL;
 	archive->priv->temp_extraction_dir = NULL;
@@ -397,7 +383,7 @@ get_local_copy_for_file (GFile *remote_file)
 	char  *temp_dir;
 	GFile *local_copy = NULL;
 
-	temp_dir = get_temp_work_dir (NULL);
+	temp_dir = _g_path_get_temp_work_dir (NULL);
 	if (temp_dir != NULL) {
 		char  *archive_name;
 		char  *local_path;
@@ -466,7 +452,7 @@ fr_archive_remove_temp_work_dir (FrArchive *archive)
 {
 	if (archive->priv->temp_dir == NULL)
 		return;
-	remove_local_directory (archive->priv->temp_dir);
+	_g_path_remove_directory (archive->priv->temp_dir);
 	g_free (archive->priv->temp_dir);
 	archive->priv->temp_dir = NULL;
 }
@@ -517,7 +503,7 @@ get_mime_type_from_content (GFile *file)
 		g_clear_error (&err);
 	}
 	else {
-		content_type = get_static_string (g_file_info_get_content_type (info));
+		content_type = _g_str_get_static (g_file_info_get_content_type (info));
 		g_object_unref (info);
 	}
 
@@ -609,7 +595,7 @@ get_mime_type_from_filename (GFile *file)
 		return NULL;
 
 	filename = g_file_get_path (file);
-	mime_type = get_mime_type_from_extension (get_file_extension (filename));
+	mime_type = get_mime_type_from_extension (_g_filename_get_extension (filename));
 	g_free (filename);
 
 	return mime_type;
@@ -622,18 +608,13 @@ create_command_from_type (FrArchive     *archive,
 		          GType          command_type,
 		          FrCommandCaps  requested_capabilities)
 {
-	char *filename;
-
 	if (command_type == 0)
 		return FALSE;
 
-	filename = g_file_get_path (archive->local_copy);
 	archive->command = FR_COMMAND (g_object_new (command_type,
 					             "process", archive->process,
-					             "filename", filename,
 					             "mime-type", mime_type,
 					             NULL));
-	g_free (filename);
 
 	if (! fr_command_is_capable_of (archive->command, requested_capabilities)) {
 		g_object_unref (archive->command);
@@ -797,16 +778,16 @@ move_here (FrArchive *archive)
 	GFile  *source, *destination, *parent_file;
 	GError *error = NULL;
 
-	content_uri = get_dir_content_if_unique (archive->priv->extraction_destination);
+	content_uri = _g_uri_get_dir_content_if_unique (archive->priv->extraction_destination);
 	if (content_uri == NULL)
 		return;
 
-	parent = remove_level_from_path (content_uri);
+	parent = _g_path_remove_level (content_uri);
 
-	if (uricmp (parent, archive->priv->extraction_destination) == 0) {
+	if (_g_uri_cmp (parent, archive->priv->extraction_destination) == 0) {
 		char *new_uri;
 
-		new_uri = get_alternative_uri_for_uri (archive->priv->extraction_destination);
+		new_uri = _g_uri_create_alternative_for_uri (archive->priv->extraction_destination);
 
 		source = g_file_new_for_uri (archive->priv->extraction_destination);
 		destination = g_file_new_for_uri (new_uri);
@@ -822,15 +803,15 @@ move_here (FrArchive *archive)
 
 		g_free (parent);
 
-		content_uri = get_dir_content_if_unique (archive->priv->extraction_destination);
+		content_uri = _g_uri_get_dir_content_if_unique (archive->priv->extraction_destination);
 		if (content_uri == NULL)
 			return;
 
-		parent = remove_level_from_path (content_uri);
+		parent = _g_path_remove_level (content_uri);
 	}
 
-	parent_parent = remove_level_from_path (parent);
-	new_content_uri = get_alternative_uri (parent_parent, file_name_from_path (content_uri));
+	parent_parent = _g_path_remove_level (parent);
+	new_content_uri = _g_uri_create_alternative (parent_parent, _g_path_get_file_name (content_uri));
 
 	source = g_file_new_for_uri (content_uri);
 	destination = g_file_new_for_uri (new_content_uri);
@@ -861,7 +842,7 @@ copy_extracted_files_done (GError   *error,
 {
 	FrArchive *archive = user_data;
 
-	remove_local_directory (archive->priv->temp_extraction_dir);
+	_g_path_remove_directory (archive->priv->temp_extraction_dir);
 	g_free (archive->priv->temp_extraction_dir);
 	archive->priv->temp_extraction_dir = NULL;
 
@@ -920,7 +901,7 @@ fr_archive_change_name (FrArchive  *archive,
 	const char *name;
 	GFile      *parent;
 
-	name = file_name_from_path (filename);
+	name = _g_path_get_file_name (filename);
 
 	parent = g_file_get_parent (archive->file);
 	g_object_unref (archive->file);
@@ -991,13 +972,13 @@ action_performed (FrCommand   *command,
 			g_print ("action_performed: ERROR!\n");
 
 			if ((archive->priv->remote_extraction) && (archive->priv->temp_extraction_dir != NULL)) {
-				remove_local_directory (archive->priv->temp_extraction_dir);
+				_g_path_remove_directory (archive->priv->temp_extraction_dir);
 				g_free (archive->priv->temp_extraction_dir);
 				archive->priv->temp_extraction_dir = NULL;
 			}
 
 			if (archive->priv->extract_here)
-				remove_directory (archive->priv->extraction_destination);
+				_g_uri_remove_directory (archive->priv->extraction_destination);
 		}
 		break;
 
@@ -1148,10 +1129,10 @@ static void
 load_local_archive (FrArchive  *archive,
 		    const char *password)
 {
-	FrCommand  *tmp_command;
+	FrCommand  *old_command;
 	const char *mime_type;
 
-	if (! g_file_query_exists (archive->file, NULL)) {
+	if (! g_file_query_exists (archive->file, archive->priv->cancellable)) {
 		fr_archive_action_completed (archive,
 					     FR_ACTION_LOADING_ARCHIVE,
 					     FR_PROC_ERROR_GENERIC,
@@ -1159,10 +1140,10 @@ load_local_archive (FrArchive  *archive,
 		return;
 	}
 
-	archive->have_permissions = check_file_permissions (archive->file, W_OK);
+	archive->have_permissions = _g_file_check_permissions (archive->file, W_OK);
 	archive->read_only = ! archive->have_permissions;
 
-	tmp_command = archive->command;
+	old_command = archive->command;
 
 	mime_type = get_mime_type_from_filename (archive->local_copy);
 	if (! create_command_to_load_archive (archive, mime_type)) {
@@ -1170,7 +1151,7 @@ load_local_archive (FrArchive  *archive,
 		if (! create_command_to_load_archive (archive, mime_type)) {
 			mime_type = get_mime_type_from_magic_numbers (archive->local_copy);
 			if (! create_command_to_load_archive (archive, mime_type)) {
-				archive->command = tmp_command;
+				archive->command = old_command;
 				archive->content_type = mime_type;
 				fr_archive_action_completed (archive,
 							     FR_ACTION_LOADING_ARCHIVE,
@@ -1181,9 +1162,9 @@ load_local_archive (FrArchive  *archive,
 		}
 	}
 
-	if (tmp_command != NULL) {
-		g_signal_handlers_disconnect_by_data (tmp_command, archive);
-		g_object_unref (tmp_command);
+	if (old_command != NULL) {
+		g_signal_handlers_disconnect_by_data (old_command, archive);
+		g_object_unref (old_command);
 	}
 
 	fr_archive_connect_to_command (archive);
@@ -1201,9 +1182,11 @@ load_local_archive (FrArchive  *archive,
 	/**/
 
 	fr_process_clear (archive->process);
-	g_object_set (archive->command, "password", password, NULL);
+	g_object_set (archive->command,
+		      "file", archive->local_copy,
+		      "password", password,
+		      NULL);
 	fr_command_list (archive->command);
-	fr_process_start (archive->process);
 }
 
 
@@ -1256,11 +1239,13 @@ copy_remote_file (FrArchive  *archive,
 {
 	XferData *xfer_data;
 
-	if (! g_file_query_exists (archive->file, NULL)) {
+	if (! g_file_query_exists (archive->file, archive->priv->cancellable)) {
 		GError *error;
+
 		error = g_error_new (G_IO_ERROR, G_IO_ERROR_NOT_FOUND, _("Archive not found"));
 		fr_archive_copy_done (archive, FR_ACTION_LOADING_ARCHIVE, error);
 		g_error_free (error);
+
 		return;
 	}
 
@@ -1306,25 +1291,6 @@ fr_archive_load (FrArchive  *archive,
 }
 
 
-gboolean
-fr_archive_load_local (FrArchive  *archive,
-		       const char *uri,
-		       const char *password)
-{
-	g_return_val_if_fail (archive != NULL, FALSE);
-
-	g_signal_emit (G_OBJECT (archive),
-		       fr_archive_signals[START],
-		       0,
-		       FR_ACTION_LOADING_ARCHIVE);
-
-	fr_archive_set_uri (archive, uri);
-	copy_remote_file (archive, password);
-
-	return TRUE;
-}
-
-
 void
 fr_archive_reload (FrArchive  *archive,
 		   const char *password)
@@ -1343,27 +1309,6 @@ fr_archive_reload (FrArchive  *archive,
 }
 
 
-void
-fr_archive_rename (FrArchive  *archive,
-		   const char *filename)
-{
-	g_return_if_fail (archive != NULL);
-
-	if (archive->is_compressed_file) {
-		/* If the archive is a compressed file we have to reload it,
-		 * because in this case the 'content' of the archive changes
-		 * too. */
-		fr_archive_load (archive, filename, NULL);
-	}
-	else {
-		if (archive->file != NULL)
-			g_object_unref (archive->file);
-		archive->file = g_file_new_for_path (filename);
-		fr_command_set_filename (archive->command, filename);
-	}
-}
-
-
 /* -- add -- */
 
 
@@ -1391,19 +1336,20 @@ create_tmp_base_dir (const char *base_dir,
 	debug (DEBUG_INFO, "base_dir: %s\n", base_dir);
 	debug (DEBUG_INFO, "dest_dir: %s\n", dest_dir);
 
-	temp_dir = get_temp_work_dir (NULL);
-	tmp = remove_level_from_path (dest_dir);
+	temp_dir = _g_path_get_temp_work_dir (NULL);
+	tmp = _g_path_remove_level (dest_dir);
 	parent_dir =  g_build_filename (temp_dir, tmp, NULL);
 	g_free (tmp);
 
 	debug (DEBUG_INFO, "mkdir %s\n", parent_dir);
-	make_directory_tree_from_path (parent_dir, 0700, NULL);
+	_g_path_make_directory_tree (parent_dir, 0700, NULL);
 	g_free (parent_dir);
 
 	dir = g_build_filename (temp_dir, "/", dest_dir, NULL);
 	debug (DEBUG_INFO, "symlink %s --> %s\n", dir, base_dir);
-	if (! symlink (base_dir, dir))
-		/* void */;
+	if (! symlink (base_dir, dir)) {
+		/* void */
+	}
 
 	g_free (dir);
 	g_free (dest_dir);
@@ -1455,7 +1401,7 @@ newer_files_only (FrArchive  *archive,
 		fullpath = g_strconcat (base_dir, "/", filename, NULL);
 		uri = g_filename_to_uri (fullpath, NULL, NULL);
 
-		if (fdata->modified >= get_file_mtime (uri)) {
+		if (fdata->modified >= _g_uri_get_file_mtime (uri)) {
 			g_free (fullpath);
 			g_free (uri);
 			continue;
@@ -1470,16 +1416,6 @@ newer_files_only (FrArchive  *archive,
 }
 
 
-void
-fr_archive_set_add_is_stoppable_func (FrArchive     *archive,
-				      FakeLoadFunc   func,
-				      gpointer       data)
-{
-	archive->priv->add_is_stoppable_func = func;
-	archive->priv->add_is_stoppable_data = data;
-}
-
-
 static gboolean
 save_list_to_temp_file (GList   *file_list,
 		        char   **list_dir,
@@ -1492,7 +1428,7 @@ save_list_to_temp_file (GList   *file_list,
 
 	if (error != NULL)
 		*error = NULL;
-	*list_dir = get_temp_work_dir (NULL);
+	*list_dir = _g_path_get_temp_work_dir (NULL);
 	*list_filename = g_build_filename (*list_dir, "file-list", NULL);
 	list_file = g_file_new_for_path (*list_filename);
 	ostream = g_file_create (list_file, G_FILE_CREATE_PRIVATE, NULL, error);
@@ -1503,7 +1439,7 @@ save_list_to_temp_file (GList   *file_list,
 		for (scan = file_list; scan != NULL; scan = scan->next) {
 			char *filename = scan->data;
 
-			filename = str_substitute (filename, "\n", "\\n");
+			filename = _g_str_substitute (filename, "\n", "\\n");
 			if ((g_output_stream_write (G_OUTPUT_STREAM (ostream), filename, strlen (filename), NULL, error) < 0)
 			    || (g_output_stream_write (G_OUTPUT_STREAM (ostream), "\n", 1, NULL, error) < 0))
 			{
@@ -1523,7 +1459,7 @@ save_list_to_temp_file (GList   *file_list,
 		error_occurred = TRUE;
 
 	if (error_occurred) {
-		remove_local_directory (*list_dir);
+		_g_path_remove_directory (*list_dir);
 		g_free (*list_dir);
 		g_free (*list_filename);
 		*list_dir = NULL;
@@ -1587,6 +1523,9 @@ fr_archive_add (FrArchive     *archive,
 	gboolean  base_dir_created = FALSE;
 	GList    *scan;
 	char     *tmp_base_dir = NULL;
+	char     *tmp_archive_dir = NULL;
+	char     *archive_filename = NULL;
+	char     *tmp_archive_filename = NULL;
 	gboolean  error_occurred = FALSE;
 
 	if (file_list == NULL)
@@ -1602,9 +1541,9 @@ fr_archive_add (FrArchive     *archive,
 		      "volume_size", volume_size,
 		      NULL);
 
-	fr_archive_stoppable (archive, fr_archive_add_is_stoppable (archive));
+	fr_archive_stoppable (archive, TRUE);
 
-	tmp_base_dir = g_strdup (base_dir);
+	/* dest_dir is the destination folder inside the archive */
 
 	if ((dest_dir != NULL) && (*dest_dir != '\0') && (strcmp (dest_dir, "/") != 0)) {
 		const char *rel_dest_dir = dest_dir;
@@ -1621,8 +1560,10 @@ fr_archive_add (FrArchive     *archive,
 			new_file_list = g_list_prepend (new_file_list, g_build_filename (rel_dest_dir, filename, NULL));
 		}
 	}
-	else
-		new_file_list = path_list_dup (file_list);
+	else {
+		tmp_base_dir = g_strdup (base_dir);
+		new_file_list = _g_string_list_dup (file_list);
+	}
 
 	/* if the command cannot update,  get the list of files that are
 	 * newer than the ones in the archive. */
@@ -1632,24 +1573,59 @@ fr_archive_add (FrArchive     *archive,
 
 		tmp_file_list = new_file_list;
 		new_file_list = newer_files_only (archive, tmp_file_list, tmp_base_dir);
-		path_list_free (tmp_file_list);
+		_g_string_list_free (tmp_file_list);
 	}
 
 	if (new_file_list == NULL) {
 		debug (DEBUG_INFO, "nothing to update.\n");
 
 		if (base_dir_created)
-			remove_local_directory (tmp_base_dir);
+			_g_path_remove_directory (tmp_base_dir);
 		g_free (tmp_base_dir);
 
 		archive->process->error.type = FR_PROC_ERROR_NONE;
 		g_signal_emit_by_name (G_OBJECT (archive->process),
 				       "done",
-				       FR_ACTION_ADDING_FILES);
+				       &archive->process->error);
 		return;
 	}
 
-	archive->command->creating_archive = ! g_file_test (archive->command->filename, G_FILE_TEST_EXISTS);
+	archive->command->creating_archive = ! g_file_query_exists (archive->local_copy, archive->priv->cancellable);
+
+	/* create the new archive in a temporary sub-directory, this allows
+	 * to cancel the operation without losing the original archive and
+	 * removing possible temporary files created by the command. */
+
+	{
+		GFile *local_copy_parent;
+		char  *archive_dir;
+		GFile *tmp_file;
+
+		/* create the new archive in a sub-folder of the original
+		 * archive this way the 'mv' command is fast. */
+
+		local_copy_parent = g_file_get_parent (archive->local_copy);
+		archive_dir = g_file_get_path (local_copy_parent);
+		tmp_archive_dir = _g_path_get_temp_work_dir (archive_dir);
+		archive_filename = g_file_get_path (archive->local_copy);
+		tmp_archive_filename = g_build_filename (tmp_archive_dir, _g_path_get_file_name (archive_filename), NULL);
+		tmp_file = g_file_new_for_path (tmp_archive_filename);
+		g_object_set (archive->command, "file", tmp_file, NULL);
+
+		if (! archive->command->creating_archive) {
+			/* copy the original archive to the new position */
+
+			fr_process_begin_command (archive->process, "cp");
+			fr_process_add_arg (archive->process, "-f");
+			fr_process_add_arg (archive->process, archive_filename);
+			fr_process_add_arg (archive->process, tmp_archive_filename);
+			fr_process_end_command (archive->process);
+		}
+
+		g_object_unref (tmp_file);
+		g_free (archive_dir);
+		g_object_unref (local_copy_parent);
+	}
 
 	fr_command_uncompress (archive->command);
 
@@ -1697,7 +1673,7 @@ fr_archive_add (FrArchive     *archive,
 			archive->process->error.gerror = g_error_copy (error);
 			g_signal_emit_by_name (G_OBJECT (archive->process),
 					       "done",
-					       FR_ACTION_ADDING_FILES);
+					       &archive->process->error);
 			g_clear_error (&error);
 			error_occurred = TRUE;
 		}
@@ -1745,12 +1721,31 @@ fr_archive_add (FrArchive     *archive,
 		g_list_free (chunks);
 	}
 
-	path_list_free (new_file_list);
+	_g_string_list_free (new_file_list);
 
 	if (! error_occurred) {
 		fr_command_recompress (archive->command);
 
-		if (base_dir_created) { /* remove the temp dir */
+		/* move the new archive to the original position */
+
+		fr_process_begin_command (archive->process, "mv");
+		fr_process_add_arg (archive->process, "-f");
+		fr_process_add_arg (archive->process, tmp_archive_filename);
+		fr_process_add_arg (archive->process, archive_filename);
+		fr_process_end_command (archive->process);
+
+		/* remove the temp sub-directory */
+
+		fr_process_begin_command (archive->process, "rm");
+		fr_process_set_working_dir (archive->process, g_get_tmp_dir());
+		fr_process_set_sticky (archive->process, TRUE);
+		fr_process_add_arg (archive->process, "-rf");
+		fr_process_add_arg (archive->process, tmp_archive_dir);
+		fr_process_end_command (archive->process);
+
+		/* remove the base dir */
+
+		if (base_dir_created) {
 			fr_process_begin_command (archive->process, "rm");
 			fr_process_set_working_dir (archive->process, g_get_tmp_dir());
 			fr_process_set_sticky (archive->process, TRUE);
@@ -1760,6 +1755,9 @@ fr_archive_add (FrArchive     *archive,
 		}
 	}
 
+	g_free (tmp_archive_filename);
+	g_free (archive_filename);
+	g_free (tmp_archive_dir);
 	g_free (tmp_base_dir);
 }
 
@@ -1775,7 +1773,6 @@ fr_archive_add_local_files (FrArchive     *archive,
 			    FrCompression  compression,
 			    guint          volume_size)
 {
-	fr_archive_stoppable (archive, TRUE);
 	fr_process_clear (archive->process);
 	fr_archive_add (archive,
 			file_list,
@@ -1856,14 +1853,14 @@ copy_remote_files (FrArchive     *archive,
 		char  *remote_uri;
 
 		local_uri = g_strconcat ("file://", tmp_dir, "/", partial_filename, NULL);
-		local_folder_uri = remove_level_from_path (local_uri);
+		local_folder_uri = _g_path_remove_level (local_uri);
 		if (g_hash_table_lookup (created_folders, local_folder_uri) == NULL) {
 			GError *error = NULL;
-			if (! ensure_dir_exists (local_folder_uri, 0755, &error)) {
+			if (! _g_uri_ensure_dir_exists (local_folder_uri, 0755, &error)) {
 				g_free (local_folder_uri);
 				g_free (local_uri);
-				gio_file_list_free (sources);
-				gio_file_list_free (destinations);
+				_g_file_list_free (sources);
+				_g_file_list_free (destinations);
 				g_hash_table_destroy (created_folders);
 
 				fr_archive_action_completed (archive,
@@ -1890,7 +1887,7 @@ copy_remote_files (FrArchive     *archive,
 
 	xfer_data = g_new0 (XferData, 1);
 	xfer_data->archive = archive;
-	xfer_data->file_list = path_list_dup (file_list);
+	xfer_data->file_list = _g_string_list_dup (file_list);
 	xfer_data->base_uri = g_strdup (base_uri);
 	xfer_data->dest_dir = g_strdup (dest_dir);
 	xfer_data->update = update;
@@ -1916,8 +1913,8 @@ copy_remote_files (FrArchive     *archive,
 			    copy_remote_files_done,
 			    xfer_data);
 
-	gio_file_list_free (sources);
-	gio_file_list_free (destinations);
+	_g_file_list_free (sources);
+	_g_file_list_free (destinations);
 }
 
 
@@ -1925,7 +1922,7 @@ static char *
 fr_archive_get_temp_work_dir (FrArchive *archive)
 {
 	fr_archive_remove_temp_work_dir (archive);
-	archive->priv->temp_dir = get_temp_work_dir (NULL);
+	archive->priv->temp_dir = _g_path_get_temp_work_dir (NULL);
 	return archive->priv->temp_dir;
 }
 
@@ -1941,7 +1938,7 @@ fr_archive_add_files (FrArchive     *archive,
 		      FrCompression  compression,
 		      guint          volume_size)
 {
-	if (uri_is_local (base_dir)) {
+	if (_g_uri_is_local (base_dir)) {
 		char *local_dir = g_filename_from_uri (base_dir, NULL, NULL);
 		fr_archive_add_local_files (archive,
 					    file_list,
@@ -2026,8 +2023,8 @@ add_with_wildcard__step2 (GList    *file_list,
 				      aww_data->compression,
 				      aww_data->volume_size);
 
-	path_list_free (file_list);
-	path_list_free (dirs_list);
+	_g_string_list_free (file_list);
+	_g_string_list_free (dirs_list);
 	add_with_wildcard_data_free (aww_data);
 }
 
@@ -2131,7 +2128,7 @@ add_directory__step2 (GList    *file_list,
 	if (archive->command->propAddCanStoreFolders)
 		file_list = g_list_concat (file_list, dir_list);
 	else
-		path_list_free (dir_list);
+		_g_string_list_free (dir_list);
 
 	if (file_list != NULL) {
 		fr_archive_add_files (ad_data->archive,
@@ -2143,7 +2140,7 @@ add_directory__step2 (GList    *file_list,
 				      ad_data->encrypt_header,
 				      ad_data->compression,
 				      ad_data->volume_size);
-		path_list_free (file_list);
+		_g_string_list_free (file_list);
 	}
 
 	add_directory_data_free (ad_data);
@@ -2242,7 +2239,7 @@ all_files_in_same_dir (GList *list)
 	if (list == NULL)
 		return FALSE;
 
-	first_basedir = remove_level_from_path (list->data);
+	first_basedir = _g_path_remove_level (list->data);
 	if (first_basedir == NULL)
 		return TRUE;
 
@@ -2250,7 +2247,7 @@ all_files_in_same_dir (GList *list)
 		char *path = scan->data;
 		char *basedir;
 
-		basedir = remove_level_from_path (path);
+		basedir = _g_path_remove_level (path);
 		if (basedir == NULL) {
 			same_dir = FALSE;
 			break;
@@ -2291,7 +2288,7 @@ add_dropped_items (DroppedItemsData *data)
 	if (all_files_in_same_dir (list)) {
 		char *first_base_dir;
 
-		first_base_dir = remove_level_from_path (list->data);
+		first_base_dir = _g_path_remove_level (list->data);
 		fr_archive_add_items (data->archive,
 				      list,
 				      first_base_dir,
@@ -2315,16 +2312,16 @@ add_dropped_items (DroppedItemsData *data)
 		char *path = scan->data;
 		char *base_dir;
 
-		if (! uri_is_dir (path))
+		if (! _g_uri_query_is_dir (path))
 			continue;
 
 		data->item_list = g_list_remove_link (list, scan);
 		if (data->item_list != NULL)
 			archive->priv->continue_adding_dropped_items = TRUE;
-		base_dir = remove_level_from_path (path);
+		base_dir = _g_path_remove_level (path);
 
 		fr_archive_add_directory (archive,
-					  file_name_from_path (path),
+					  _g_path_get_file_name (path),
 					  base_dir,
 					  data->dest_dir,
 					  data->update,
@@ -2345,12 +2342,12 @@ add_dropped_items (DroppedItemsData *data)
 		char  *first_basedir;
 		GList *only_names_list = NULL;
 
-		first_basedir = remove_level_from_path (list->data);
+		first_basedir = _g_path_remove_level (list->data);
 
 		for (scan = list; scan; scan = scan->next) {
 			char *name;
 
-			name = g_uri_unescape_string (file_name_from_path (scan->data), NULL);
+			name = g_uri_unescape_string (_g_path_get_file_name (scan->data), NULL);
 			only_names_list = g_list_prepend (only_names_list, name);
 		}
 
@@ -2364,7 +2361,7 @@ add_dropped_items (DroppedItemsData *data)
 				      data->compression,
 				      data->volume_size);
 
-		path_list_free (only_names_list);
+		_g_string_list_free (only_names_list);
 		g_free (first_basedir);
 
 		return;
@@ -2374,7 +2371,9 @@ add_dropped_items (DroppedItemsData *data)
 	 * files without path info. FIXME: doesn't work with remote files. */
 
 	fr_archive_stoppable (archive, FALSE);
+	archive->command->creating_archive = ! g_file_query_exists (archive->local_copy, archive->priv->cancellable);
 	g_object_set (archive->command,
+		      "file", archive->local_copy,
 		      "password", data->password,
 		      "encrypt_header", data->encrypt_header,
 		      "compression", data->compression,
@@ -2387,8 +2386,8 @@ add_dropped_items (DroppedItemsData *data)
 		char  *basedir;
 		GList *singleton;
 
-		basedir = remove_level_from_path (fullpath);
-		singleton = g_list_prepend (NULL, (char*)file_name_from_path (fullpath));
+		basedir = _g_path_remove_level (fullpath);
+		singleton = g_list_prepend (NULL, (char*)_g_path_get_file_name (fullpath));
 		fr_command_add (archive->command,
 				NULL,
 				singleton,
@@ -2401,7 +2400,7 @@ add_dropped_items (DroppedItemsData *data)
 	fr_command_recompress (archive->command);
 	fr_process_start (archive->process);
 
-	path_list_free (data->item_list);
+	_g_string_list_free (data->item_list);
 	data->item_list = NULL;
 }
 
@@ -2431,7 +2430,7 @@ fr_archive_add_dropped_items (FrArchive     *archive,
 	/* FIXME: make this check for all the add actions */
 	archive_uri = g_file_get_uri (archive->file);
 	for (scan = item_list; scan; scan = scan->next) {
-		if (uricmp (scan->data, archive_uri) == 0) {
+		if (_g_uri_cmp (scan->data, archive_uri) == 0) {
 			g_free (archive_uri);
 			fr_archive_action_completed (archive,
 						     FR_ACTION_ADDING_FILES,
@@ -2473,7 +2472,7 @@ file_is_in_subfolder_of (const char *filename,
 	for (scan = folder_list; scan; scan = scan->next) {
 		char *folder_in_list = (char*) scan->data;
 
-		if (path_in_path (folder_in_list, filename))
+		if (_g_path_is_parent_of (folder_in_list, filename))
 			return TRUE;
 	}
 
@@ -2600,16 +2599,80 @@ fr_archive_remove (FrArchive     *archive,
 		   GList         *file_list,
 		   FrCompression  compression)
 {
+	char *tmp_archive_dir = NULL;
+	char *archive_filename = NULL;
+	char *tmp_archive_filename = NULL;
+
 	g_return_if_fail (archive != NULL);
 
 	if (archive->read_only)
 		return;
 
-	fr_archive_stoppable (archive, FALSE);
+	fr_archive_stoppable (archive, TRUE);
+	archive->command->creating_archive = FALSE;
 	g_object_set (archive->command, "compression", compression, NULL);
+
+	/* create the new archive in a temporary sub-directory, this allows
+	 * to cancel the operation without losing the original archive and
+	 * removing possible temporary files created by the command. */
+
+	{
+		GFile *local_copy_parent;
+		char  *archive_dir;
+		GFile *tmp_file;
+
+		/* create the new archive in a sub-folder of the original
+		 * archive this way the 'mv' command is fast. */
+
+		local_copy_parent = g_file_get_parent (archive->local_copy);
+		archive_dir = g_file_get_path (local_copy_parent);
+		tmp_archive_dir = _g_path_get_temp_work_dir (archive_dir);
+		archive_filename = g_file_get_path (archive->local_copy);
+		tmp_archive_filename = g_build_filename (tmp_archive_dir, _g_path_get_file_name (archive_filename), NULL);
+		tmp_file = g_file_new_for_path (tmp_archive_filename);
+		g_object_set (archive->command, "file", tmp_file, NULL);
+
+		if (! archive->command->creating_archive) {
+			/* copy the original archive to the new position */
+
+			fr_process_begin_command (archive->process, "cp");
+			fr_process_add_arg (archive->process, "-f");
+			fr_process_add_arg (archive->process, archive_filename);
+			fr_process_add_arg (archive->process, tmp_archive_filename);
+			fr_process_end_command (archive->process);
+		}
+
+		g_object_unref (tmp_file);
+		g_free (archive_dir);
+		g_object_unref (local_copy_parent);
+	}
+
+	/* uncompress, delete and recompress */
+
 	fr_command_uncompress (archive->command);
 	delete_from_archive (archive, file_list);
 	fr_command_recompress (archive->command);
+
+	/* move the new archive to the original position */
+
+	fr_process_begin_command (archive->process, "mv");
+	fr_process_add_arg (archive->process, "-f");
+	fr_process_add_arg (archive->process, tmp_archive_filename);
+	fr_process_add_arg (archive->process, archive_filename);
+	fr_process_end_command (archive->process);
+
+	/* remove the temp sub-directory */
+
+	fr_process_begin_command (archive->process, "rm");
+	fr_process_set_working_dir (archive->process, g_get_tmp_dir());
+	fr_process_set_sticky (archive->process, TRUE);
+	fr_process_add_arg (archive->process, "-rf");
+	fr_process_add_arg (archive->process, tmp_archive_dir);
+	fr_process_end_command (archive->process);
+
+	g_free (tmp_archive_filename);
+	g_free (archive_filename);
+	g_free (tmp_archive_dir);
 }
 
 
@@ -2821,7 +2884,7 @@ compute_base_path (const char *base_dir,
 
 	if (junk_paths) {
 		if (can_junk_paths)
-			new_path = g_strdup (file_name_from_path (path));
+			new_path = g_strdup (_g_path_get_file_name (path));
 		else
 			new_path = g_strdup (path);
 
@@ -2916,7 +2979,7 @@ file_list_contains_files_in_this_dir (GList      *file_list,
 	for (scan = file_list; scan; scan = scan->next) {
 		char *filename = scan->data;
 
-		if (path_in_path (dirname, filename))
+		if (_g_path_is_parent_of (dirname, filename))
 			return TRUE;
 	}
 
@@ -2938,7 +3001,7 @@ remove_files_contained_in_this_dir (GList *file_list,
 		if (strncmp (dirname, filename, dirname_l) != 0)
 			break;
 
-		if (path_in_path (dirname, filename)) {
+		if (_g_path_is_parent_of (dirname, filename)) {
 			GList *next = scan->next;
 
 			file_list = g_list_remove_link (file_list, scan);
@@ -2975,6 +3038,7 @@ fr_archive_extract_to_local (FrArchive  *archive,
 	g_return_if_fail (archive != NULL);
 
 	fr_archive_stoppable (archive, TRUE);
+	g_object_set (archive->command, "file", archive->local_copy, NULL);
 
 	/* if a command supports all the requested options use
 	 * fr_command_extract directly. */
@@ -3031,7 +3095,7 @@ fr_archive_extract_to_local (FrArchive  *archive,
 			g_list_free (filtered);
 
 		if (file_list_created)
-			path_list_free (file_list);
+			_g_string_list_free (file_list);
 
 		return;
 	}
@@ -3076,7 +3140,7 @@ fr_archive_extract_to_local (FrArchive  *archive,
 		if (! junk_paths)
 			filename = archive_list_filename;
 		else
-			filename = file_name_from_path (archive_list_filename);
+			filename = _g_path_get_file_name (archive_list_filename);
 
 		if ((destination[strlen (destination) - 1] == '/')
 		    || (filename[0] == '/'))
@@ -3091,7 +3155,7 @@ fr_archive_extract_to_local (FrArchive  *archive,
 		if (! archive->command->propExtractCanSkipOlder
 		    && skip_older
 		    && g_file_test (dest_filename, G_FILE_TEST_EXISTS)
-		    && (fdata->modified < get_file_mtime_for_path (dest_filename)))
+		    && (fdata->modified < _g_path_get_file_mtime (dest_filename)))
 			continue;
 
 		if (! archive->command->propExtractCanAvoidOverwrite
@@ -3107,14 +3171,14 @@ fr_archive_extract_to_local (FrArchive  *archive,
 		debug (DEBUG_INFO, "All files got filtered, nothing to do.\n");
 
 		if (extract_all)
-			path_list_free (file_list);
+			_g_string_list_free (file_list);
 		return;
 	}
 
 	if (move_to_dest_dir) {
 		char *temp_dir;
 
-		temp_dir = get_temp_work_dir (destination);
+		temp_dir = _g_path_get_temp_work_dir (destination);
 		extract_from_archive (archive,
 				      filtered,
 				      temp_dir,
@@ -3156,7 +3220,7 @@ fr_archive_extract_to_local (FrArchive  *archive,
 	if (filtered != NULL)
 		g_list_free (filtered);
 	if (file_list_created)
-		path_list_free (file_list);
+		_g_string_list_free (file_list);
 }
 
 
@@ -3176,9 +3240,9 @@ fr_archive_extract (FrArchive  *archive,
 	g_free (archive->priv->temp_extraction_dir);
 	archive->priv->temp_extraction_dir = NULL;
 
-	archive->priv->remote_extraction = ! uri_is_local (destination);
+	archive->priv->remote_extraction = ! _g_uri_is_local (destination);
 	if (archive->priv->remote_extraction) {
-		archive->priv->temp_extraction_dir = get_temp_work_dir (NULL);
+		archive->priv->temp_extraction_dir = _g_path_get_temp_work_dir (NULL);
 		fr_archive_extract_to_local (archive,
 					     file_list,
 					     archive->priv->temp_extraction_dir,
@@ -3328,7 +3392,10 @@ fr_archive_test (FrArchive  *archive,
 {
 	fr_archive_stoppable (archive, TRUE);
 
-	g_object_set (archive->command, "password", password, NULL);
+	g_object_set (archive->command,
+		      "file", archive->local_copy,
+		      "password", password,
+		      NULL);
 	fr_process_clear (archive->process);
 	fr_command_set_n_files (archive->command, 0);
 	fr_command_test (archive->command);
@@ -3337,7 +3404,7 @@ fr_archive_test (FrArchive  *archive,
 
 
 gboolean
-uri_is_archive (const char *uri)
+_g_uri_is_archive (const char *uri)
 {
 	GFile      *file;
 	const char *mime_type;
