@@ -33,6 +33,7 @@
 #include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-7z.h"
+#include "rar-utils.h"
 
 static void fr_command_7z_class_init  (FrCommand7zClass *class);
 static void fr_command_7z_init        (FrCommand        *afile);
@@ -99,7 +100,7 @@ list__process_line (char     *line,
 			int         ver_len;
 			char        version[256];
 
-			ver_start = eat_spaces (line + 14);
+			ver_start = _g_str_eat_spaces (line + 14);
 			ver_len = strchr (ver_start, ' ') - ver_start;
 			strncpy (version, ver_start, ver_len);
 			version[ver_len] = 0;
@@ -130,10 +131,10 @@ list__process_line (char     *line,
 			else {
 				fdata = p7z_comm->fdata;
 				if (fdata->dir)
-					fdata->name = dir_name_from_path (fdata->full_path);
+					fdata->name = _g_path_get_dir_name (fdata->full_path);
 				else
-					fdata->name = g_strdup (file_name_from_path (fdata->full_path));
-				fdata->path = remove_level_from_path (fdata->full_path);
+					fdata->name = g_strdup (_g_path_get_file_name (fdata->full_path));
+				fdata->path = _g_path_remove_level (fdata->full_path);
 				fr_command_add_file (comm, fdata);
 				p7z_comm->fdata = NULL;
 			}
@@ -194,11 +195,11 @@ list__process_line (char     *line,
 static void
 fr_command_7z_begin_command (FrCommand *comm)
 {
-	if (is_program_in_path ("7z"))
+	if (_g_program_is_in_path ("7z"))
 		fr_process_begin_command (comm->process, "7z");
-	else if (is_program_in_path ("7za"))
+	else if (_g_program_is_in_path ("7za"))
 		fr_process_begin_command (comm->process, "7za");
-	else if (is_program_in_path ("7zr"))
+	else if (_g_program_is_in_path ("7zr"))
 		fr_process_begin_command (comm->process, "7zr");
 }
 
@@ -234,7 +235,7 @@ list__begin (gpointer data)
 static void
 fr_command_7z_list (FrCommand  *comm)
 {
-	rar_check_multi_vomule (comm);
+	rar_check_multi_volume (comm);
 
 	fr_process_set_out_line_func (comm->process, list__process_line, comm);
 
@@ -253,28 +254,29 @@ fr_command_7z_list (FrCommand  *comm)
 }
 
 
-static char Progress_Message[4196];
-static char Progress_Filename[4096];
-
-
 static void
 parse_progress_line (FrCommand  *comm,
 		     const char *prefix,
-		     const char *message_prefix,
+		     const char *message_format,
 		     const char *line)
 {
 	int prefix_len;
 
 	prefix_len = strlen (prefix);
 	if (strncmp (line, prefix, prefix_len) == 0) {
-		double fraction;
+		if (comm->n_files > 1) {
+			fr_command_progress (comm, (double) ++comm->n_file / (comm->n_files + 1));
+		}
+		else {
+			char  filename[4196];
+			char *msg;
 
-		strcpy (Progress_Filename, line + prefix_len);
-		sprintf (Progress_Message, "%s%s", message_prefix, file_name_from_path (Progress_Filename));
-		fr_command_message (comm, Progress_Message);
+			strcpy (filename, line + prefix_len);
+			msg = g_strdup_printf (message_format, filename, NULL);
+			fr_command_message (comm, msg);
 
-		fraction = (double) ++comm->n_file / (comm->n_files + 1);
-		fr_command_progress (comm, fraction);
+			g_free (msg);
+		}
 	}
 }
 
@@ -286,15 +288,19 @@ process_line__add (char     *line,
 	FrCommand *comm = FR_COMMAND (data);
 
 	if ((comm->volume_size > 0) && (strncmp (line, "Creating archive ", 17) == 0)) {
-		char *volume_filename;
+		char  *volume_filename;
+		GFile *volume_file;
 
 		volume_filename = g_strconcat (comm->filename, ".001", NULL);
-		fr_command_set_multi_volume (comm, volume_filename);
+		volume_file = g_file_new_for_path (volume_filename);
+		fr_command_set_multi_volume (comm, volume_file);
+
+		g_object_unref (volume_file);
 		g_free (volume_filename);
 	}
 
 	if (comm->n_files != 0)
-		parse_progress_line (comm, "Compressing  ", _("Adding file: "), line);
+		parse_progress_line (comm, "Compressing  ", _("Adding \"%s\""), line);
 }
 
 
@@ -325,8 +331,8 @@ fr_command_7z_add (FrCommand     *comm,
 		fr_process_add_arg_concat (comm->process, "-w", base_dir, NULL);
 	}
 
-	if (is_mime_type (comm->mime_type, "application/zip")
-	    || is_mime_type (comm->mime_type, "application/x-cbz"))
+	if (_g_mime_type_matches (comm->mime_type, "application/zip")
+	    || _g_mime_type_matches (comm->mime_type, "application/x-cbz"))
 	{
 		fr_process_add_arg (comm->process, "-tzip");
 		fr_process_add_arg (comm->process, "-mem=AES128");
@@ -362,7 +368,7 @@ fr_command_7z_add (FrCommand     *comm,
 		break;
 	}
 
-	if (is_mime_type (comm->mime_type, "application/x-ms-dos-executable"))
+	if (_g_mime_type_matches (comm->mime_type, "application/x-ms-dos-executable"))
 		fr_process_add_arg (comm->process, "-sfx");
 
 	if (comm->volume_size > 0)
@@ -392,7 +398,7 @@ fr_command_7z_delete (FrCommand  *comm,
 	fr_process_add_arg (comm->process, "d");
 	fr_process_add_arg (comm->process, "-bd");
 	fr_process_add_arg (comm->process, "-y");
-	if (is_mime_type (comm->mime_type, "application/x-ms-dos-executable"))
+	if (_g_mime_type_matches (comm->mime_type, "application/x-ms-dos-executable"))
 		fr_process_add_arg (comm->process, "-sfx");
 
 	if (from_file != NULL)
@@ -415,7 +421,7 @@ process_line__extract (char     *line,
 	FrCommand *comm = FR_COMMAND (data);
 
 	if (comm->n_files != 0)
-		parse_progress_line (comm, "Extracting  ", _("Extracting file: "), line);
+		parse_progress_line (comm, "Extracting  ", _("Extracting \"%s\""), line);
 }
 
 
@@ -553,22 +559,22 @@ fr_command_7z_get_capabilities (FrCommand  *comm,
 	FrCommandCap capabilities;
 
 	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES;
-	if (! is_program_available ("7za", check_command) && ! is_program_available ("7zr", check_command) && ! is_program_available ("7z", check_command))
+	if (! _g_program_is_available ("7za", check_command) && ! _g_program_is_available ("7zr", check_command) && ! _g_program_is_available ("7z", check_command))
 		return capabilities;
 
-	if (is_mime_type (mime_type, "application/x-7z-compressed")) {
+	if (_g_mime_type_matches (mime_type, "application/x-7z-compressed")) {
 		capabilities |= FR_COMMAND_CAN_READ_WRITE | FR_COMMAND_CAN_CREATE_VOLUMES;
-		if (is_program_available ("7z", check_command))
+		if (_g_program_is_available ("7z", check_command))
 			capabilities |= FR_COMMAND_CAN_ENCRYPT | FR_COMMAND_CAN_ENCRYPT_HEADER;
 	}
-	else if (is_mime_type (mime_type, "application/x-7z-compressed-tar")) {
+	else if (_g_mime_type_matches (mime_type, "application/x-7z-compressed-tar")) {
 		capabilities |= FR_COMMAND_CAN_READ_WRITE;
-		if (is_program_available ("7z", check_command))
+		if (_g_program_is_available ("7z", check_command))
 			capabilities |= FR_COMMAND_CAN_ENCRYPT | FR_COMMAND_CAN_ENCRYPT_HEADER;
 	}
-	else if (is_program_available ("7z", check_command)) {
-		if (is_mime_type (mime_type, "application/x-rar")
-		    || is_mime_type (mime_type, "application/x-cbr"))
+	else if (_g_program_is_available ("7z", check_command)) {
+		if (_g_mime_type_matches (mime_type, "application/x-rar")
+		    || _g_mime_type_matches (mime_type, "application/x-cbr"))
 		{
 			if (! check_command || g_file_test ("/usr/lib/p7zip/Codecs/Rar29.so", G_FILE_TEST_EXISTS))
 				capabilities |= FR_COMMAND_CAN_READ;
@@ -576,21 +582,21 @@ fr_command_7z_get_capabilities (FrCommand  *comm,
 		else
 			capabilities |= FR_COMMAND_CAN_READ;
 
-		if (is_mime_type (mime_type, "application/x-cbz")
-		    || is_mime_type (mime_type, "application/x-ms-dos-executable")
-		    || is_mime_type (mime_type, "application/zip"))
+		if (_g_mime_type_matches (mime_type, "application/x-cbz")
+		    || _g_mime_type_matches (mime_type, "application/x-ms-dos-executable")
+		    || _g_mime_type_matches (mime_type, "application/zip"))
 		{
 			capabilities |= FR_COMMAND_CAN_WRITE | FR_COMMAND_CAN_ENCRYPT;
 		}
 	}
-	else if (is_program_available ("7za", check_command)) {
-		if (is_mime_type (mime_type, "application/vnd.ms-cab-compressed")
-		    || is_mime_type (mime_type, "application/zip"))
+	else if (_g_program_is_available ("7za", check_command)) {
+		if (_g_mime_type_matches (mime_type, "application/vnd.ms-cab-compressed")
+		    || _g_mime_type_matches (mime_type, "application/zip"))
 		{
 			capabilities |= FR_COMMAND_CAN_READ;
 		}
 
-		if (is_mime_type (mime_type, "application/zip"))
+		if (_g_mime_type_matches (mime_type, "application/zip"))
 			capabilities |= FR_COMMAND_CAN_WRITE;
 	}
 
@@ -606,9 +612,9 @@ static const char *
 fr_command_7z_get_packages (FrCommand  *comm,
 			    const char *mime_type)
 {
-	if (is_mime_type (mime_type, "application/x-rar"))
+	if (_g_mime_type_matches (mime_type, "application/x-rar"))
 		return PACKAGES ("p7zip,p7zip-rar");
-	else if (is_mime_type (mime_type, "application/zip") || is_mime_type (mime_type, "application/vnd.ms-cab-compressed"))
+	else if (_g_mime_type_matches (mime_type, "application/zip") || _g_mime_type_matches (mime_type, "application/vnd.ms-cab-compressed"))
 		return PACKAGES ("p7zip,p7zip-full");
 	else
 		return PACKAGES ("p7zip");
