@@ -65,7 +65,7 @@ mktime_from_string (char *month,
 	tm.tm_mday = atoi (mday);
 	if (strchr (year, ':') != NULL) {
 		char **fields = g_strsplit (year, ":", 2);
-        	if (n_fields (fields) == 2) {
+        	if (g_strv_length (fields) == 2) {
 	        	time_t      now;
         		struct tm  *now_tm;
 
@@ -99,28 +99,28 @@ list__process_line (char     *line,
 	fdata = file_data_new ();
 
 #ifdef __sun
-	fields = split_line (line, 9);
+	fields = _g_str_split_line (line, 9);
 	fdata->size = g_ascii_strtoull (fields[4], NULL, 10);
 	fdata->modified = mktime_from_string (fields[5], fields[6], fields[8]);
 	g_strfreev (fields);
 
-	name_field = get_last_field (line, 10);
+	name_field = _g_str_get_last_field (line, 10);
 #else /* !__sun */
 	/* Handle char and block device files */
 	if ((line[0] == 'c') || (line[0] == 'b')) {
-		fields = split_line (line, 9);
+		fields = _g_str_split_line (line, 9);
 		ofs = 1;
 		fdata->size = 0;
 		/* FIXME: We should also specify the content type */
 	}
 	else {
-		fields = split_line (line, 8);
+		fields = _g_str_split_line (line, 8);
 		fdata->size = g_ascii_strtoull (fields[4], NULL, 10);
 	}
 	fdata->modified = mktime_from_string (fields[5+ofs], fields[6+ofs], fields[7+ofs]);
 	g_strfreev (fields);
 
-	name_field = get_last_field (line, 9+ofs);
+	name_field = _g_str_get_last_field (line, 9+ofs);
 #endif /* !__sun */
 
 	fields = g_strsplit (name_field, " -> ", 2);
@@ -156,10 +156,10 @@ list__process_line (char     *line,
 	g_strfreev (fields);
 
 	if (fdata->dir)
-		fdata->name = dir_name_from_path (fdata->full_path);
+		fdata->name = _g_path_get_dir_name (fdata->full_path);
 	else
-		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
-	fdata->path = remove_level_from_path (fdata->full_path);
+		fdata->name = g_strdup (_g_path_get_file_name (fdata->full_path));
+	fdata->path = _g_path_remove_level (fdata->full_path);
 
 	if (*fdata->name == 0)
 		file_data_free (fdata);
@@ -197,7 +197,7 @@ fr_command_cpio_extract (FrCommand *comm,
 	if (dest_dir != NULL)
                 fr_process_set_working_dir (comm->process, dest_dir);
 	fr_process_add_arg (comm->process, "-c");
-	
+
 	cmd = g_string_new ("cpio -idu --no-absolute-filenames ");
 	for (scan = file_list; scan; scan = scan->next) {
 		char *filepath = scan->data;
@@ -218,7 +218,6 @@ fr_command_cpio_extract (FrCommand *comm,
 	g_string_free (cmd, TRUE);
 
 	fr_process_end_command (comm->process);
-	fr_process_start (comm->process);
 }
 
 
@@ -240,7 +239,7 @@ fr_command_cpio_get_capabilities (FrCommand  *comm,
 	FrCommandCap capabilities;
 
 	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES;
-	if (is_program_available ("cpio", check_command))
+	if (_g_program_is_available ("cpio", check_command))
 		capabilities |= FR_COMMAND_CAN_READ;
 
 	return capabilities;
