@@ -120,12 +120,12 @@ split_line_lha (char *line)
 		line += strlen ("[unknown]");
 	}
 
-	scan = eat_spaces (line);
+	scan = _g_str_eat_spaces (line);
 	for (; i < n_fields; i++) {
 		field_end = strchr (scan, ' ');
 		if (field_end != NULL) {
 			fields[i] = g_strndup (scan, field_end - scan);
-			scan = eat_spaces (field_end);
+			scan = _g_str_eat_spaces (field_end);
 		}
 	}
 
@@ -149,10 +149,10 @@ get_last_field_lha (char *line)
 	if (strncmp (line, "[unknown]", 9) == 0)
 		n--;
 
-	field = eat_spaces (line);
+	field = _g_str_eat_spaces (line);
 	for (i = 0; i < n; i++) {
 		field = strchr (field, ' ');
-		field = eat_spaces (field);
+		field = _g_str_eat_spaces (field);
 	}
 
 	return field;
@@ -195,11 +195,11 @@ process_line (char     *line,
 
 	fdata->dir = line[0] == 'd';
 	if (fdata->dir)
-		fdata->name = dir_name_from_path (fdata->full_path);
+		fdata->name = _g_path_get_dir_name (fdata->full_path);
 	else
-		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
+		fdata->name = g_strdup (_g_path_get_file_name (fdata->full_path));
 
-	fdata->path = remove_level_from_path (fdata->full_path);
+	fdata->path = _g_path_remove_level (fdata->full_path);
 
 	if (*fdata->name == 0)
 		file_data_free (fdata);
@@ -318,7 +318,7 @@ fr_command_lha_get_capabilities (FrCommand  *comm,
 	FrCommandCap capabilities;
 
 	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES;
-	if (is_program_available ("lha", check_command))
+	if (_g_program_is_available ("lha", check_command))
 		capabilities |= FR_COMMAND_CAN_READ_WRITE;
 
 	return capabilities;
