@@ -34,6 +34,7 @@
 #include "fr-command.h"
 #include "fr-command-rar.h"
 #include "fr-error.h"
+#include "rar-utils.h"
 
 static void fr_command_rar_class_init  (FrCommandRarClass *class);
 static void fr_command_rar_init        (FrCommand         *afile);
@@ -47,7 +48,7 @@ static FrCommandClass *parent_class = NULL;
 static gboolean
 have_rar (void)
 {
-	return is_program_in_path ("rar");
+	return _g_program_is_in_path ("rar");
 }
 
 
@@ -123,7 +124,7 @@ process_line (char     *line,
 
 		/* read file info. */
 
-		fields = split_line (line, 6);
+		fields = _g_str_split_line (line, 6);
 		if (g_strv_length (fields) < 6) {
 			/* wrong line format, treat this line as a filename line */
 			g_strfreev (fields);
@@ -155,11 +156,11 @@ process_line (char     *line,
 
 					g_free (tmp);
 
-					fdata->name = dir_name_from_path (fdata->full_path);
+					fdata->name = _g_path_get_dir_name (fdata->full_path);
 					fdata->dir = TRUE;
 				}
 				else
-					fdata->name = g_strdup (file_name_from_path (fdata->full_path));
+					fdata->name = g_strdup (_g_path_get_file_name (fdata->full_path));
 
 				fr_command_add_file (comm, fdata);
 				rar_comm->fdata = NULL;
@@ -190,7 +191,7 @@ process_line (char     *line,
 		}
 
 		fdata->link = NULL;
-		fdata->path = remove_level_from_path (fdata->full_path);
+		fdata->path = _g_path_remove_level (fdata->full_path);
 	}
 	else {
 
@@ -228,7 +229,7 @@ list__begin (gpointer data)
 static void
 fr_command_rar_list (FrCommand  *comm)
 {
-	rar_check_multi_vomule (comm);
+	rar_check_multi_volume (comm);
 
 	fr_process_set_out_line_func (comm->process, process_line, comm);
 
@@ -253,42 +254,43 @@ fr_command_rar_list (FrCommand  *comm)
 }
 
 
-static char Progress_Message[4196];
-static char Progress_Filename[4096];
-
-
 static void
 parse_progress_line (FrCommand  *comm,
 		     const char *prefix,
-		     const char *message_prefix,
+		     const char *message_format,
 		     const char *line)
 {
 	int prefix_len;
 
 	prefix_len = strlen (prefix);
 	if (strncmp (line, prefix, prefix_len) == 0) {
-		double  fraction;
-		int     len;
-		char   *b_idx;
+		if (comm->n_files > 1) {
+			fr_command_progress (comm, (double) ++comm->n_file / (comm->n_files + 1));
+		}
+		else {
+			char  filename[4096];
+			char *b_idx;
+			int   len;
+			char *msg;
 
-		strcpy (Progress_Filename, line + prefix_len);
+			strcpy (filename, line + prefix_len);
 
-		/* when a new volume is created a sequence of backspaces is
-		 * issued, remove the backspaces from the filename */
-		b_idx = strchr (Progress_Filename, '\x08');
-		if (b_idx != NULL)
-			*b_idx = 0;
+			/* when a new volume is created a sequence of backspaces is
+			 * issued, remove the backspaces from the filename */
+			b_idx = strchr (filename, '\x08');
+			if (b_idx != NULL)
+				*b_idx = 0;
 
-		/* remove the OK at the end of the filename */
-		len = strlen (Progress_Filename);
-		if ((len > 5) && (strncmp (Progress_Filename + len - 5, "  OK ", 5) == 0))
-			Progress_Filename[len - 5] = 0;
+			/* remove the OK at the end of the filename */
+			len = strlen (filename);
+			if ((len > 5) && (strncmp (filename + len - 5, "  OK ", 5) == 0))
+				filename[len - 5] = 0;
 
-		sprintf (Progress_Message, "%s%s", message_prefix, file_name_from_path (Progress_Filename));
-		fr_command_message (comm, Progress_Message);
+			msg = g_strdup_printf (message_format, _g_path_get_file_name (filename), NULL);
+			fr_command_message (comm, msg);
 
-		fraction = (double) ++comm->n_file / (comm->n_files + 1);
-		fr_command_progress (comm, fraction);
+			g_free (msg);
+		}
 	}
 }
 
@@ -307,11 +309,15 @@ process_line__add (char     *line,
 		if ((comm->volume_size > 0)
 		    && g_regex_match_simple ("^.*\\.part(0)*2\\.rar$", uri, G_REGEX_CASELESS, 0))
 		{
-			char *volume_filename;
+			char  *volume_filename;
+			GFile *volume_file;
 
 			volume_filename = g_strdup (archive_filename);
 			volume_filename[strlen (volume_filename) - 5] = '1';
-			fr_command_set_multi_volume (comm, volume_filename);
+			volume_file = g_file_new_for_path (volume_filename);
+			fr_command_set_multi_volume (comm, volume_file);
+
+			g_object_unref (volume_file);
 			g_free (volume_filename);
 		}
 		fr_command_working_archive (comm, uri);
@@ -321,7 +327,7 @@ process_line__add (char     *line,
 	}
 
 	if (comm->n_files != 0)
-		parse_progress_line (comm, "Adding    ", _("Adding file: "), line);
+		parse_progress_line (comm, "Adding    ", _("Adding \"%s\""), line);
 }
 
 
@@ -399,7 +405,7 @@ process_line__delete (char     *line,
 	}
 
 	if (comm->n_files != 0)
-		parse_progress_line (comm, "Deleting ", _("Removing file: "), line);
+		parse_progress_line (comm, "Deleting ", _("Removing \"%s\""), line);
 }
 
 
@@ -448,7 +454,7 @@ process_line__extract (char     *line,
 	}
 
 	if (comm->n_files != 0)
-		parse_progress_line (comm, "Extracting  ", _("Extracting file: "), line);
+		parse_progress_line (comm, "Extracting  ", _("Extracting \"%s\""), line);
 }
 
 
@@ -609,9 +615,9 @@ fr_command_rar_get_capabilities (FrCommand  *comm,
 	FrCommandCap capabilities;
 
 	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES | FR_COMMAND_CAN_ENCRYPT | FR_COMMAND_CAN_ENCRYPT_HEADER;
-	if (is_program_available ("rar", check_command))
+	if (_g_program_is_available ("rar", check_command))
 		capabilities |= FR_COMMAND_CAN_READ_WRITE | FR_COMMAND_CAN_CREATE_VOLUMES;
-	else if (is_program_available ("unrar", check_command))
+	else if (_g_program_is_available ("unrar", check_command))
 		capabilities |= FR_COMMAND_CAN_READ;
 
 	/* multi-volumes are read-only */
