@@ -180,12 +180,12 @@ migrate_options_directory (void)
 
 	old_directory_path = get_home_relative_path (".gnome2/file-roller/options");
 	old_directory = g_file_new_for_path (old_directory_path);
-	new_directory = get_user_config_subdirectory (ADD_FOLDER_OPTIONS_DIR, FALSE);
+	new_directory = _g_file_new_user_config_subdir (ADD_FOLDER_OPTIONS_DIR, FALSE);
 	if (g_file_query_exists (old_directory, NULL) && ! g_file_query_exists (new_directory, NULL)) {
 		GFile *parent;
 
 		parent = g_file_get_parent (new_directory);
-		if (make_directory_tree (parent, 0700, NULL))
+		if (_g_file_make_directory_tree (parent, 0700, NULL))
 			g_file_move (old_directory, new_directory, 0, NULL, NULL, NULL, NULL);
 
 		g_object_unref (parent);
@@ -221,7 +221,7 @@ fr_registered_command_new (GType command_type)
 		FrMimeTypeCap      *cap;
 		FrMimeTypePackages *packages;
 
-		mime_type = get_static_string (mime_types[i]);
+		mime_type = _g_str_get_static (mime_types[i]);
 
 		cap = g_new0 (FrMimeTypeCap, 1);
 		cap->mime_type = mime_type;
@@ -443,7 +443,7 @@ get_mime_type_from_extension (const char *ext)
 		if (file_ext_type[i].ext == NULL)
 			continue;
 		if (strcasecmp (ext, file_ext_type[i].ext) == 0)
-			return get_static_string (file_ext_type[i].mime_type);
+			return _g_str_get_static (file_ext_type[i].mime_type);
 	}
 
 	return NULL;
@@ -459,7 +459,7 @@ get_archive_filename_extension (const char *filename)
 	if (filename == NULL)
 		return NULL;
 
-	ext = get_file_extension (filename);
+	ext = _g_filename_get_extension (filename);
 	if (ext == NULL)
 		return NULL;
 
@@ -607,6 +607,9 @@ initialize_data (void)
 					       g_free,
 					       NULL);
 
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
+					   PKG_DATA_DIR G_DIR_SEPARATOR_S "icons");
+
 	migrate_options_directory ();
 	register_commands ();
 	compute_supported_archive_types ();
@@ -620,7 +623,7 @@ command_done (CommandData *cdata)
 	if (cdata == NULL)
 		return;
 
-	if ((cdata->temp_dir != NULL) && path_is_dir (cdata->temp_dir)) {
+	if ((cdata->temp_dir != NULL) && _g_path_query_is_dir (cdata->temp_dir)) {
 		char *argv[4];
 
 		argv[0] = "rm";
@@ -637,7 +640,7 @@ command_done (CommandData *cdata)
 	g_free (cdata->command);
 	if (cdata->app != NULL)
 		g_object_unref (cdata->app);
-	path_list_free (cdata->file_list);
+	_g_string_list_free (cdata->file_list);
 	g_free (cdata->temp_dir);
 	if (cdata->process != NULL)
 		g_object_unref (cdata->process);
