@@ -122,8 +122,6 @@ void        fr_window_archive_save_as           (FrWindow      *window,
 						 gboolean       encrypt_header,
 						 guint          volume_size);
 void        fr_window_archive_reload            (FrWindow      *window);
-void        fr_window_archive_rename            (FrWindow      *window,
-						 const char    *filename);
 void        fr_window_archive_add_files         (FrWindow      *window,
 						 GList         *file_list, /* GFile list */
 						 gboolean       update);
@@ -283,7 +281,10 @@ void        fr_window_use_progress_dialog         (FrWindow   *window,
 
 /* batch mode procedures. */
 
-void       fr_window_new_batch                    (FrWindow      *window);
+void       fr_window_new_batch                    (FrWindow      *window,
+						   const char    *title);
+const char *
+           fr_window_get_batch_title              (FrWindow      *window);
 void       fr_window_set_current_batch_action     (FrWindow      *window,
 						   FrBatchActionType  action,
 						   void          *data,
